#!/usr/bin/python3 -u
# encoding=utf-8
# baut Websocket-Verbindung zu MIYO auf und verschickt per UDP alle Werte an Loxone
# eingehende UDP-Befehle werden umgewandelt und an MIYO versandt
# Oliver Engel; 29.01.2019 - 22.06.2019; 02.10.2019; 31.03.2020
# MIYOplugin@phantasoft.de - http://miyoplugin.phantasoft.de

try:
  import sys
  import websocket
  import json
  import socket
  import logging
  import requests
  import time
  from collections import OrderedDict
  import logging.handlers
  import configparser
  import os
except ImportError as error:
  errstr = str(error).replace("'","")
  #print("import failed: "+str(error))
  print("import failed: "+errstr+"\n")
  #logPrint("import failed: "+errstr)
  exit(1)

# adjust here if necessary (but defaults to starting dir or LB-Plugin-dir
CONFIG_FILE = ""
#CONFIG_FILE = "/root/miyoplugin.conf"

circuit = []
circuits = []

boolList = ["valveInitialClose" , "valveStatus" , "lowPower" , "otauPossible" , "winterMode" , "charging" , "chargingLess" , "openValve" , "reachable" , "sunWithinWeek" , "irrigationNecessary" , "irrigationPossible" , "winter" , "automaticMode" , "considerMower" , "valveStaggering" , "irrigation" , "rain" , "block" , "running" , "mower", "cloudAllowed" , "linkMode" , "irrigationDelayForecast" , "allowed" , "valveStaggering" , "externBlock" , "forecast"]
timeList = ["lastUpdate" , "lastChargingTime" , "lastIrrigationEnd" , "lastIrrigationDuration" , "lastResetTime" , "lastIrrigationStart" , "irrigationNextStart" , "irrigationNextEnd"]
strList = ["version" , "uuid" , "description" , "name" , "firmware"]

prgname = "MIYOplugin"
prgver = "v0.05"
maxdev = 10

wsconnected=False
updcount=0

def allPrint(s):
  s = "### "+s+" ###"
  if loglog:
    logger.info(s)
  if rawlog:
    rawlogger.info(s)
  if sndlog:
    sndlogger.info(s)
  print(s)

def logPrint(s):
  s = "### "+s+" ###"
  if loglog:
    logger.info(s)
  print(s)

def rawPrint(s):
  s = "### "+s+" ###"
  if rawlog:
    rawlogger.info(s)
  print(s)

def sndPrint(s):
  s = "### "+s+" ###"
  if sndlog:
    sndlogger.info(s)
  print(s)

def getUDPname(ID):
# get UDPname from deviceID (messages coming from MIYO to plugin)
  UDPname = "unknown"
  for i in range(len(circuits)):
    if str(circuits[i][2]) == ID:
      UDPname = circuits[i][4]
      break
  return UDPname

def getUDPcname(ID):
# get UDP-name from circuitID (messages coming from MIYO to plugin)
  UDPname = "unknown"
  for i in range(len(circuits)):
    if str(circuits[i][0]) == ID:
      UDPname = circuits[i][1]
      break
  return UDPname

def getUDPcid(ID):
# get circuitID from UDPname (messages coming from plugin to MIYO)
  UDPname = "unknown"
  for i in range(len(circuits)):
    if str(circuits[i][1]) == ID:
      UDPname = circuits[i][0]
      break
  return UDPname

def getSection(ID):
  # get sectionName from UDPname or CircuitId
  UDPname = "unknown"
  idlen = len(ID)
  if idlen == 38 and circuitid[0] == "{" and circuitid[idlen-1] == "}" :
    j = 0
  else :
    j = 1
  for i in range(len(circuits)):
    if str(circuits[i][j]) == ID:
      UDPname = circuits[i][5]
      break
  return UDPname
                                  
def loxTime(wert):
  # MIYO sendet UTC-Zeit; hier Umrechnung in Lokalzeit und dann nach Loxone
  wert=int(wert)
  if wert > 31536000:              # groesser als 1 Jahr?
    offset = -time.timezone
    if time.localtime(wert)[8]:
      wert = wert + offset + 3600  # 7200
    else:
      wert = wert + offset         # 3600
  return wert-1230768000 if LOX_TIME and wert >= 1230768000 else wert

def sendUDP(UDPstr):
  #print("sende UDP ("+LOX_IP+":"+LOX_PORT+"): "+UDPstr)
  sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # UDP
  sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
  try:
    #sock.sendto(bytes(UDPstr, "utf-8"), (LOX_IP, int(LOX_PORT)))
    sock.sendto(bytes(UDPstr, "ISO-8859-1"), (LOX_IP, int(LOX_PORT)))
  except:
    logPrint("sendUDP to "+LOX_IP+":"+LOX_PORT+" im except!")
  if loglog :
    logger.info(UDPstr)

def send_to_MIYO(msgcmd,msgopt="",circuitid=""):
  msg = "{\"id\":0,\"method\":\""+msgcmd+"\"";
  if msgopt != "":
    msg += ",\"params\":{";
    if circuitid != "":
      # falls Klarnamen als CircuitId angegeben, rueckwandeln ueber getUDPcid
      idlen = len(circuitid)
      if idlen != 38 or circuitid[0] != "{" or circuitid[idlen-1] != "}" :
        circuitid = getUDPcid(circuitid)
      msg += "\"circuitId\":\""+circuitid+"\"";
      if ";" in msgopt :
        data=msgopt.split(";")
      else :
        data=msgopt.split(" ")
      for part in data:
        line=part.split("=")
        # Bool-Werte sind ohne "" zu senden!
        if line[0] in boolList:
          msg += ",\""+line[0]+"\":"+line[1];
        else:
          msg += ",\""+line[0]+"\":\""+line[1]+"\"";
      msg += "}";
  msg += ",\"apiKey\":\"" +API_KEY + "\"";
  msg += "}";
  if sndlog :
    sndlogger.info(msg)
  ws.send(msg)

def getCubeIP() :
  cubeIP = "not found - try again!"
  msg = 'M-SEARCH * HTTP/1.1\r\nHOST:239.255.255.250:1900\r\nST:upnp:rootdevice\r\nMX:2\r\nMAN:"ssdp:discover"\r\n\r\n'
  # Set up UDP socket
  s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
  s.settimeout(5)  # was 2
  s.sendto(bytes(msg,"utf-8"), ('239.255.255.250', 1900) )
  found = False
  try:
    while not found :
      data, addr = s.recvfrom(65507)
      if "miyocube" in str(data.decode()) :
        cubeIP = str(addr[0])
        found = True
  except socket.timeout:
    pass
  s.close()
  return cubeIP

def getAPIKey() :
  APIKey = "no connection to MIYO-Cube"
  try:
    jdata = json.loads(requests.get("http://"+MIYO_IP+"/api/link").text, object_pairs_hook=OrderedDict)
    # jetzt Nachricht pruefen und API-Key ausschneiden
    if jdata["status"] == "error" :
      APIKey = jdata["error"]
      #logPrint("could not get API-Key: "+APIKey)
    else :
      APIKey  = jdata["apiKey"]
  except:
    #logPrint("could not get API-Key (except): "+APIKey)
    pass
  return APIKey

def getCircuit(circuitid="") :
  response = ""
  if circuitid != "" :
    myurl="http://"+MIYO_IP+"/api/"+"Circuit.status"+"?apiKey="+API_KEY+"&circuitId="+circuitid
  else :
    myurl="http://"+MIYO_IP+"/api/"+"Circuit.all"+"?apiKey="+API_KEY
  try:
    response = requests.get(myurl).text
  except:
    response = "can not get circuit(s) from ip address "+MIYO_IP
    pass
  return response

def on_message(ws, inmessage):
  if rawlog :
    rawlogger.info(inmessage)
  UDPstr=""
  jdata = json.loads(inmessage, object_pairs_hook=OrderedDict)
  if jdata["id"] == 0 :
    try:
      for device in jdata["params"]["devices"]:
        jdev=device[0:38]
        UDPstr="SID="+getUDPname(jdev)
        try:
          name=jdata["params"]["devices"][device]["name"]
          UDPstr+=" name=\""+str(name)+"\""
        except:
          pass
        try:
          lastUpdate=jdata["params"]["devices"][device]["lastUpdate"]
          lastUpdate=loxTime(lastUpdate)
          UDPstr+=" lastUpdate="+str(lastUpdate)
        except:
          pass
        try:
          firmware=jdata["params"]["devices"][device]["firmware"]
          UDPstr+=" firmware=\""+str(firmware)+"\""
        except:
          pass
        for i in range(0, 64):
          try:
            jtype = str(jdata["params"]["devices"][device]["stateTypes"][str(i)]["type"])
            jvalue = str(jdata["params"]["devices"][device]["stateTypes"][str(i)]["value"])
            if jtype in boolList :
              jvalue = int(jvalue.lower() == "true")
            elif jtype in timeList :
              jvalue = loxTime(jvalue)
            elif jtype in strList :
              jvalue = "\"" + jvalue + "\""
            UDPstr += " " + str(jtype) + "=" + str(jvalue)
          except:
            pass
        # jetzt UDPstr versenden
        sendUDP(UDPstr)
    except:
      pass
    # hier weiter mit Circuit-Eingangsmeldung
    try:
      for circuit in jdata["params"]["circuits"]:
        UDPstr = "SID="+getUDPcname(circuit)
        UDPstr += " name=\""+str(jdata["params"]["circuits"][circuit]["name"])+"\""
        try:
          for jtype in jdata["params"]["circuits"][circuit]["params"]:
            jvalue = str(jdata["params"]["circuits"][circuit]["params"][jtype])
            if jtype in boolList :
              jvalue = int(jvalue.lower() == 'true')
            elif jtype in timeList :
              jvalue = loxTime(jvalue)
            elif jtype in strList :
              jvalue = "\"" + jvalue + "\""
            UDPstr += " "+str(jtype)+"="+str(jvalue)
        except:
          pass
        for i in range(0, 64):
          try:
            jtype = str(jdata["params"]["circuits"][circuit]["stateTypes"][str(i)]["type"])
            jvalue = str(jdata["params"]["circuits"][circuit]["stateTypes"][str(i)]["value"])
            if jtype in boolList :
              jvalue = int(jvalue.lower() == "true")
            elif jtype in timeList :
              jvalue = loxTime(jvalue)
            elif jtype in strList :
              jvalue = "\"" + jvalue + "\""
            UDPstr += " " + str(jtype) + "=" + str(jvalue)
          except:
            pass
        sendUDP(UDPstr);
    except:
      pass
    # weder devices noch circuits
    try:
      if UDPstr == "" :
        UDPstr = "SID=MIYOcentral"
        for jtype in jdata["params"]:
          jvalue = jdata["params"][jtype]
          if jtype in boolList :
            jvalue = int(str(jvalue).lower() == 'true')
          elif jtype in timeList :
            jvalue = loxTime(jvalue)
          elif jtype in strList :
            jvalue = "\"" + jvalue + "\""
          UDPstr += " " + str(jtype) + "=" + str(jvalue)
        sendUDP(UDPstr)
    except:
      pass
    # messages per UDP versenden
    try:
      UDPstr = "SID=MIYOmessage"
      UDPstr += " " + "\"" + str(jdata["message"]) + "\""
      sendUDP(UDPstr)
    except:
      pass
  elif jdata["id"] == -1 :
    if jdata["notification"] == "Weather.changed" :
      UDPstr += "SID=MIYOcentral"
      for jtype in jdata["params"]["weather"]:
        jvalue = jdata["params"]["weather"][jtype]
        if jtype in boolList :
          jvalue = int(str(jvalue).lower() == 'true')
        elif jtype in timeList :
          jvalue = loxTime(jvalue)
        elif jtype in strList :
          jvalue = "\"" + jvalue + "\""
        UDPstr += " " + str(jtype) + "=" + str(jvalue)
    elif jdata["notification"] == "Device.updated" :
      UDPstr += "SID=MIYOcentral updated=1"
      try:
        global updcount
        updcount += 1
        if updcount == int(UPD_INTERVAL) :
          send_to_MIYO("Device.all")
          updcount=0
      except:
        if sndlog :
          sndPrint("updated-except!")
        pass
    elif jdata["notification"] == "Device.stateChanged" :
      try:
        jdev = jdata["params"]["deviceId"]
        UDPstr="SID="+getUDPname(jdev)
        jtype = str(jdata["params"]["type"])
        jvalue = str(jdata["params"]["value"])
        if jtype in boolList :
          jvalue = int(jvalue.lower() == 'true')
        elif jtype in timeList :
          jvalue = loxTime(jvalue)
        elif jtype in strList :
          jvalue = "\"" + jvalue + "\""
        UDPstr += " " + str(jtype) + "=" + str(jvalue)
      except:
        pass
    elif jdata["notification"] == "Circuit.edited" :
      jcir = jdata["params"]["circuit"]["id"]
      UDPstr="SID="+getUDPcname(jcir)
      UDPstr += " edited=1 "
      UDPstr += "name=\""+str(jdata["params"]["circuit"]["name"])+"\""
      try:
        for jtype in jdata["params"]["circuit"]["params"]:
          jvalue = str(jdata["params"]["circuit"]["params"][jtype])
          if jtype in boolList :
            jvalue = int(jvalue.lower() == 'true')
          elif jtype in timeList :
            jvalue = loxTime(jvalue)
          elif jtype in strList :
            jvalue = "\"" + jvalue + "\""
          UDPstr += " "+str(jtype)+"="+str(jvalue)
      except:
        pass
    elif jdata["notification"] == "Circuit.stateChanged" :
      jcir = jdata["params"]["circuitId"]
      UDPstr = "SID="+getUDPcname(jcir)
      #UDPstr = "SID=MIYOcircuit"
      jtype = str(jdata["params"]["type"])
      jvalue = str(jdata["params"]["value"])
      if jtype in boolList :
        jvalue = int(jvalue.lower() == 'true')
      elif jtype in timeList :
        jvalue = loxTime(jvalue)
      elif jtype in strList :
        jvalue = "\"" + jvalue + "\""
      UDPstr += " "+str(jtype)+"="+str(jvalue)
    elif jdata["notification"] == "Circuit.updated" :
      UDPstr = "SID=MIYOcircuit updated=1"
      send_to_MIYO("Circuit.all")
    else :
      if loglog :
        logger.info("jdata: ",str(jdata))
    # jetzt UDPstr versenden
    sendUDP(UDPstr);
  else :
    logPrint("on_message: weder Null noch minus Eins!")

def on_error(ws, error):
  print(error)
  if loglog :
    logger.critical(error)
  global wsconnected
  #wsconnected=False
  if wsconnected :
    logPrint("websocket broken")
    sendUDP("SID=MIYOcentral running=0")
    wsconnected=False
    # damit der UDP-Socket geschlossen wird, ein Pseudo-Datagram schicken
    try:
      #ssock.sendto(bytes("STOP", "utf-8"), (LB_IP, int(LB_PORT)))
      ssock.sendto(bytes("STOP", "ISO-8859-1"), (LB_IP, int(LB_PORT)))
    except:
      logPrint("can not send the stop-datagram to close the socket")
      pass

def on_close(ws):
  global wsconnected
  if wsconnected :
    logPrint("websocket closed")
    sendUDP("SID=MIYOcentral running=0")
    wsconnected=False
  # damit der UDP-Socket geschlossen wird, ein Pseudo-Datagram schicken
  try:
    #ssock.sendto(bytes("STOP", "utf-8"), (LB_IP, int(LB_PORT)))
    ssock.sendto(bytes("STOP", "ISO-8859-1"), (LB_IP, int(LB_PORT)))
  except:
    logPrint("can not send the stop-datagram to close the socket")
    pass

def on_open(ws):
  logPrint("websocket on "+MIYO_IP+" opened")
  sendUDP("SID=MIYOcentral running=1")
  # Setzen des externBlock wenn im Config-File gespeichert EXT_REM
  if EXT_REM :
    last = ""
    #print("len: "+str(len(circuits)))
    for i in range(0,len(circuits)):
      #print(str(i)+"*"+last+"* "+str(circuits[i][6]))
      if circuits[i][6] and last != str(circuits[i][0]):
        #sndPrint("recover last blocking-state for circuit "+str(circuits[i][1]))
        logPrint("recover last blocking-state for circuit "+str(circuits[i][1]))
        send_to_MIYO("Extern.block","block=true",circuits[i][0])
        last = str(circuits[i][0])
  send_to_MIYO("Device.all")
  send_to_MIYO("Circuit.all")
  send_to_MIYO("System.status")
  #global wsconnected
  #wsconnected=True

formatter = logging.Formatter('%(asctime)s.%(msecs)03d %(levelname)s %(message)s',datefmt="%d.%m.%Y %H:%M:%S")

def setup_logger(name, log_file, level=logging.INFO, format=formatter):
  #handler = logging.FileHandler(log_file)
  handler = logging.handlers.WatchedFileHandler(log_file) # neu ab v0.3
  handler.setFormatter(format)
  logger = logging.getLogger(name)
  logger.setLevel(level)
  logger.addHandler(handler)
  return logger

def checkUDPport(IP,PORT):
  udpopen = False
  ssock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # Internet, UDP
  try:
    ssock.bind((IP, int(PORT)))
    ssock.close()
    #print("port is usable")
    udpopen = True
  except OSError as msg:
    #print('could not open socket')
    pass
  return udpopen

# ------------------------------------------------------------
# main
# ------------------------------------------------------------
# Option abfragen; moeglich sind: -getCubeIP, -getCircuit, -getAPIKey und -checkLBPort
try:
  option = sys.argv[1]
except:
  option = ""
  pass

# das Config-File ist fuer getCubeIP nicht noetig, daher zuerst:
if option == '-getCubeIP':
  print(getCubeIP())
  sys.exit(0)

# Config-File finden
# search the Config-File for MIYOplugin - defaults to start-path
CFG_FILE = os.path.dirname(__file__)+"/miyoplugin.conf"
if os.path.isfile(CFG_FILE):
  CONFIG_FILE = CFG_FILE
elif "LBPCONFIG" in os.environ:
  CFG_FILE = os.environ.get("LBPCONFIG")+"/miyoplugin/miyoplugin.conf"
  if os.path.isfile(CFG_FILE):
    CONFIG_FILE = CFG_FILE

# Konfiguration einlesen
config = configparser.RawConfigParser(inline_comment_prefixes='#')
config.optionxform = str
config.read(CONFIG_FILE, encoding='ISO-8859-1')

MIYO_IP = config.get('Config','MIYO_IP',fallback='MIYO_IP')
LOX_IP = config.get('Config','LOX_IP',fallback='LOX_IP')
#LOX_PORT = config.getint('Config','LOX_PORT',fallback='LOX_PORT')
LOX_PORT = config.get('Config','LOX_PORT',fallback='LOX_PORT')
LB_IP = config.get('Config','LB_IP',fallback='LB_IP')
#LB_PORT = config.getint('Config','LB_PORT',fallback='LB_PORT')
LB_PORT = config.get('Config','LB_PORT',fallback='LB_PORT')
API_KEY = config.get('Config','API_KEY',fallback='API_KEY')
LOX_TIME = config.getboolean('Config','LOX_TIME',fallback=False)
#UPD_INTERVAL = config.getint('Config','UPD_INTERVAL',fallback=10)
UPD_INTERVAL = config.get('Config','UPD_INTERVAL',fallback='10')
EXT_REM = config.getboolean('Config','EXT_REM',fallback=False)

logfile = config.get('Logging','logfile',fallback='')
rawfile = config.get('Logging','rawfile',fallback='')
sndfile = config.get('Logging','sndfile',fallback='')

loglog = True if logfile != '' else False
rawlog = True if rawfile != '' else False
sndlog = True if sndfile != '' else False

# first file logger
if loglog :
  try:
    logger = setup_logger('std_logger',logfile,format=formatter)
  except:
    print("### can not log std_logger to "+logfile)
    loglog = False
    pass

# raw-Logger
myformatter = logging.Formatter('%(asctime)s.%(msecs)03d %(message)s',datefmt="%d.%m.%Y %H:%M:%S")
if rawlog :
  try:
    rawlogger = setup_logger('raw_logger',rawfile,format=myformatter)
  except:
    logPrint("can not log raw_logger to "+rawfile)
    rawlog = False
    pass

# send-Logger
if sndlog :
  try:
    sndlogger = setup_logger('snd_logger',sndfile,format=formatter)
  except:
    logPrint("can not log snd_logger to "+sndfile)
    sndlog = False
    pass

# fuer getCircuit und getAPIKey sind CircuitIDs und IP-Adressen noetig, daher erst nach Einlesen der Config
if option == '-getCircuit':
  try:
    param = sys.argv[2]
  except:
    param = ""
    pass
  print(getCircuit(param))
  sys.exit(0)
elif option == '-getAPIKey':
  print(getAPIKey())
  sys.exit(0)
elif option == '-createConfig':
  # Parameter targetip targetport myport
  if len(sys.argv) >= 7:
    MIYO_IP = getCubeIP()
    LOX_IP = sys.argv[2]
    LOX_PORT = sys.argv[3]
    LB_PORT = sys.argv[4]
    LOX_TIME = sys.argv[5]
    EXT_REM = sys.argv[6]
  else:
    print("you have to call -createConfig with additional parameters LOX_IP LOX_PORT LB_PORT LOX_TIME")
    sys.exit(0)
  API_KEY = getAPIKey()
  if not config.has_section("Config") :
    config.add_section('Config')
  config.set('Config', 'MIYO_IP', MIYO_IP)
  config.set('Config', 'LOX_IP', LOX_IP)
  config.set('Config', 'LOX_PORT', LOX_PORT)
  config.set('Config', 'LB_PORT', LB_PORT)
  config.set('Config', 'API_KEY', API_KEY)
  config.set('Config', 'LOX_TIME', LOX_TIME)
  config.set('Config', 'EXT_REM', EXT_REM)
  try:
    i = 0
    jdata = json.loads(getCircuit(), object_pairs_hook=OrderedDict)
    for circuit in jdata["params"]["circuits"]:
      i+=1
      j=0
      cir_id = str(jdata["params"]["circuits"][circuit]["id"])
      cir_name = str(jdata["params"]["circuits"][circuit]["name"])
      # replace all spaces with "_"
      #cir_name = cir_name.replace(" ","_")
      cir_mid = "Circuit-"+str(i)
      if not config.has_section(cir_mid) :
        config.add_section(cir_mid)
      config.set(cir_mid, 'circuitid', cir_id)
      config.set(cir_mid, 'circuitname', cir_name)
      for valve in jdata["params"]["circuits"][circuit]["valves"]:
        j += 1
        v_id = str(jdata["params"]["circuits"][circuit]["valves"][valve]["valve"])
        v_ipv6 = str(jdata["params"]["circuits"][circuit]["valves"][valve]["valveData"]["ipv6"])
        config.set(cir_mid, 'valveid-'+str(j), v_id)
        config.set(cir_mid, 'valvename-'+str(j), cir_name+"-V"+str(j))
        config.set(cir_mid, 'valveipv6-'+str(j), v_ipv6)
      s_id = str(jdata["params"]["circuits"][circuit]["sensor"])
      s_ipv6 = str(jdata["params"]["circuits"][circuit]["sensorData"]["ipv6"])
      config.set(cir_mid, 'sensorname-1', cir_name+"-S1")
      config.set(cir_mid, 'sensorid-1', s_id)
      config.set(cir_mid, 'sensoripv6-1', s_ipv6)
  except:
    if API_KEY[0] != "{":
      logPrint("createConfig: did not get a API-key - "+API_KEY)
    else:
      logPrint("createConfig: getCircuit() did not get a correct response")
    pass
  with open(CONFIG_FILE, 'w') as CONFIG_FILE:
    config.write(CONFIG_FILE)
  sys.exit(0)
elif option == '-checkLBPort':
  if len(sys.argv) >= 3 and (checkUDPport("",int(sys.argv[2])) or int(sys.argv[2]) == int(LB_PORT)):
    print("ok")
  else:
    print("failed")
  sys.exit(0)

for i in range(1, maxdev+1):
  section = 'Circuit-'+str(i)
  if config.has_section(section):
    circuitid = config.get(section,'circuitid')
    circuitname = config.get(section,'circuitname')
    circuitblock = config.getboolean(section,'circuitblock',fallback=False)
    #if EXT_REM and circuitblock :
      #print(section+" with ID "+circuitid+" is blocked: "+str(circuitblock))
    for j in range(1, maxdev+1):
      option = 'valveid-'+str(j)
      if config.has_option(section,option):
        valveid = config.get(section,option)
        option = 'valvename-'+str(j)
        valvename = config.get(section,option)
        circuit = [circuitid, circuitname, valveid, "Valve", valvename, section, circuitblock]
        circuits.append(circuit)
        #print("appended!"+str(circuit))

      option = 'sensorid-'+str(j)
      if config.has_option(section,option):
        sensorid = config.get(section,option)
        option = 'sensorname-'+str(j)
        sensorname = config.get(section,option)
        circuit = [circuitid, circuitname, sensorid, "Sensor", sensorname, section, circuitblock]
        circuits.append(circuit)
        #print("appended!"+str(circuit))

allPrint(prgname+" "+prgver+" started")

# Websocket und zusaetzlich den UDP-Server starten
import threading
try:
  ws = websocket.WebSocketApp("ws://"+MIYO_IP+":3810", on_message = on_message, on_error = on_error, on_close = on_close)
  ws.on_open = on_open
  wst = threading.Thread(target=ws.run_forever)
  wst.daemon = True
  wst.start()
  timeout = 3
  while not ws.sock.connected and timeout:
    time.sleep(1)
    timeout -= 1
    if (timeout == 0):
      wsconnected = False
      logPrint("can not open websocket on "+MIYO_IP+" (initial)")
    else:
      wsconnected = True
except:
  logPrint("can not open websocket on "+MIYO_IP)
  wsconnected = False
  pass

if wsconnected:
  #print("LB_IP: "+str(LB_IP)+" LB_PORT: "+str(LB_PORT))
  ssock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) # Internet, UDP
  try:
    ssock.bind((LB_IP, int(LB_PORT)))
    #if sndlog :
    #  sndPrint("local UDP-socket "+str(LB_PORT)+" bound")
    logPrint("local UDP-socket "+str(LB_PORT)+" bound")
  except:
    logPrint("can not open UDP-socket "+str(LB_PORT)+" on ip address "+str(LB_IP))
    pass

  logPrint("remote UDP: "+LOX_IP+":"+LOX_PORT)

  while wsconnected:
    try:
      sdata, saddr = ssock.recvfrom(1024) # buffer size is 1024 bytes
      r_dgram = str(sdata.decode())
      r_addr = str(saddr[0])
      r_port = str(saddr[1])

      # eingehende Nachricht pruefen
      anzahl = r_dgram.count(',')+1           # Anzahl der Felder
      if anzahl > 0:
        data=r_dgram.split(",")
        if data[0] == "SID=MIYOplugin":       # ist eine zu behandelnde Nachricht
          if anzahl >= 2 :
            cmd = str(data[1])
            opt = ""
            cir = ""
            if anzahl >= 3 :
              opt = str(data[2])
              if anzahl >= 4 :
                cir = str(data[3])
              else:
                cir = str(circuits[0][0])
            send_to_MIYO(cmd,opt,cir)
            # repoll the current state of Extern-methods
            if "Extern." in cmd and cmd != "Extern.status":
              send_to_MIYO("Extern.status","",cir)
              # wennn EXT_REM dann extBlock-Status im Config-File abspeichern
              #logPrint("Circuit: "+circuits[0])
              #EXT_REM = True
              if EXT_REM and "Extern.block" in cmd and "block" in opt:
              # cir kann leer sein oder den UDP-Namen oder die echte ID enthalten
                cir = getSection(cir)
                #for i in range (0,6):
                #  print(str(i)+": "+str(circuits[0][i]))
                #print("cir: "+cir)
                if "true" in opt:
                  # write true in ConfigFile
                  #logPrint("write to ConfigFile "+cir+" True")
                  config.set(cir, 'circuitblock', 'True')
                else:
                  # write false in ConfigFile
                  #logPrint("write to ConfigFile "+cir+" False")
                  config.set(cir, 'circuitblock', 'False')
                # Config-File schreiben
                try:
                  config.write(open(CONFIG_FILE, 'w'))
                except:
                  print("error while writing Config-File!")
                  pass
      #
    except:
      if sndlog :
        sndPrint("except in while wsconnected!")
      break

  try:
    wst.do_run = False
    ssock.close()
  except:
    pass
  #if sndlog :
  #  sndPrint("UDP-socket closed")
  logPrint("local UDP-socket "+str(LB_PORT)+" closed")
allPrint(prgname+" "+prgver+" stopped")

