#!/usr/bin/perl

# This is a sample Script file
# It does not much:
#   * Loading configuration
#   * including header.htmlfooter.html
#   * and showing a message to the user.
# That's all.
# Now it does a bit more ...
# Oliver Engel; 29.01.2019 - 22.06.2019; 02.10.2019; 31.03.2020
# MIYOplugin@phantasoft.de - http://miyoplugin.phantasoft.de

use File::HomeDir;
use CGI qw/:standard/;
#use Config::Simple;
use Cwd 'abs_path';
use IO::Socket::INET;
use HTML::Entities;
use String::Escape qw( unquotemeta );
use LoxBerry::System;
use JSON;
use Encode qw(encode decode);
use warnings;
use strict;
no strict "refs"; # we need it for template system
use CGI::Carp qw/fatalsToBrowser/;

my  $myDebug = 0; # set to 1 to enable debug-output in Apache-Log
my  $prgname = "MIYOplugin";
my  $prgver = LoxBerry::System::pluginversion();
my  $maxdev = 10;
my  $home = File::HomeDir->my_home;
our $lang;
my  $installfolder;
my  $cfg;
my  $conf;
our $template_title;
our $namef;
our $value;
our %query;
our $phrase;
our $phraseplugin;
our $languagefile;
our $languagefileplugin;
our $cache;
our $savedata;
our $MSselectlist;
our $miniserver;

our $enabled;
our $helptext;
our $helplink;
our $restartMsg;
our $logdir;
our $linkvorlage = "";

# hier MIYO-Vars
our $MIYO_IP;
our $LOX_IP;
our $LOX_PORT;
our $LB_IP;
our $LB_PORT;
our $API_KEY;
our $LOX_TIME;
our $UPD_INTERVAL;
our $EXT_REM;
our $logfile;
our $rawfile;
our $sndfile;
our $CubeConfig;

our @kreis;
our @kreise = ["Kreis","target","Name","ID"];
our @ventil;
our @ventile = ["Kreis","target","Name","ID"];
our @sensor;
our @sensoren = ["Kreis","target","Name","ID"];
our $marker;

sub logPrint {
  my $outstr = $_[0];
  my $timestamp = localtime(time);
  if ($outstr ne "") {
    print STDERR "$timestamp $prgname: $outstr\n";
  } else {
    print STDERR "\n";
  }
}

sub debugPrint {
  my $str = $_[0];
  chomp($str);
  if ($myDebug) { logPrint($str); }
}

sub clearString($) {
  my $string = shift;
  $string =~ s/#[^|]*//g if defined($string);
  $string =~ s/\s+$// if defined($string);
  $string = "" if not defined $string;
  return $string;
}

# start the cgi-script
debugPrint("cgi start");

# ---------------------------------------
# Read Settings
# ---------------------------------------
debugPrint("read settings start");
my $gencfg = "$home/config/system/general.cfg";
$cfg = new Config::Simple($gencfg);
eval { $installfolder = $cfg->param("BASE.INSTALLFOLDER") };
if ($@) { die "error while getting installfolder from $gencfg" }
eval { $lang = $cfg->param("BASE.LANG") };
if ($@) { die "error while getting lang from $gencfg" }

my $LBPDATA = $ENV{'LBPDATA'};
my $LBPCONFIG = $ENV{'LBPCONFIG'};
my $LBPBIN = $ENV{'LBPBIN'};
$logdir = $ENV{'LBPLOG'}."/$lbpplugindir/";
my $lbip = LoxBerry::System::get_localip();

our $Yes = "Yes";
our $No = "No";
if ($lang eq "de") {
  $Yes = "Ja";
  $No = "Nein";
}
debugPrint("  prgver: ".$prgver);
debugPrint("  home: ".$home);
debugPrint("  genCFG: ".$gencfg);
debugPrint("  installfolder: ".$installfolder);
debugPrint("  lang: ".$lang);
debugPrint("  LBPDATA: ".$LBPDATA);
debugPrint("  LBPCONFIG: ".$LBPCONFIG);
debugPrint("  LBPBIN: ".$LBPBIN);
debugPrint("  logdir: ".$logdir);
debugPrint("  lbip: ".$lbip);
debugPrint("read settings done");

print "Content-Type: text/html\n\n";

# ---------------------------------------
# Parse URL
# ---------------------------------------
debugPrint("parse url start");
foreach (split(/&/,$ENV{"QUERY_STRING"}))
{
  ($namef,$value) = split(/=/,$_,2);
  $namef =~ tr/+/ /;
  $namef =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $value =~ tr/+/ /;
  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $query{$namef} = $value;
}
debugPrint("parse url done");

# ---------------------------------------
# Set parameters coming in - GET over POST
# ---------------------------------------
# alle Parameter einlesen - Kreise gibt es zu diesem Zeitpunkt aber noch nicht!
debugPrint("parse parameters coming in start");
my $p;
my @param_names = param();
foreach $p (@param_names) {
  $value = param($p);
  my $str = substr $p,0,2;
  if ($str eq "c_" or $str eq "v_" or $str eq "s_") {
    ${$p} = $value;
#  } else {
#    print STDERR "$p: $value\n";
  }
}

if ( param('doNothing') )   {
  #Sicherheitsabfrage abgebrochen; weiter geht's
}

if ( !$query{'MIYO_IP'} )   { if ( param('MIYO_IP')  ) { $MIYO_IP = quotemeta(param('MIYO_IP'));         }
else { $MIYO_IP = $MIYO_IP;  } } else { $MIYO_IP = quotemeta($query{'MIYO_IP'});   }

if ( !$query{'LOX_IP'} )   { if ( param('LOX_IP')  ) { $LOX_IP = quotemeta(param('LOX_IP'));         }
else { $LOX_IP = $LOX_IP;  } } else { $LOX_IP = quotemeta($query{'LOX_IP'});   }

if ( param('miniserver') )   {
  $LOX_IP = quotemeta(param('miniserver'));
}

if ( !$query{'LOX_PORT'} )   { if ( param('LOX_PORT')  ) { $LOX_PORT = quotemeta(param('LOX_PORT'));         }
else { $LOX_PORT = $LOX_PORT;  } } else { $LOX_PORT = quotemeta($query{'LOX_PORT'});   }

if ( !$query{'LB_IP'} )   { if ( param('LB_IP')  ) { $LB_IP = quotemeta(param('LB_IP'));         }
else { $LB_IP = $LB_IP;  } } else { $LB_IP = quotemeta($query{'LB_IP'});   }

if ( !$query{'LB_PORT'} )   { if ( param('LB_PORT')  ) { $LB_PORT = quotemeta(param('LB_PORT'));         }
else { $LB_PORT = $LB_PORT;  } } else { $LB_PORT = quotemeta($query{'LB_PORT'});   }

if ( !$query{'API_KEY'} )   { if ( param('API_KEY')  ) { $API_KEY = quotemeta(param('API_KEY'));         }
else { $API_KEY = $API_KEY;  } } else { $API_KEY = quotemeta($query{'API_KEY'});   }

if ( !$query{'LOX_TIME'} )   { if ( param('LOX_TIME')  ) { $LOX_TIME = quotemeta(param('LOX_TIME'));         }
else { $LOX_TIME = $LOX_TIME;  } } else { $LOX_TIME = quotemeta($query{'LOX_TIME'});   }

if ( !$query{'UPD_INTERVAL'} )   { if ( param('UPD_INTERVAL')  ) { $UPD_INTERVAL = quotemeta(param('UPD_INTERVAL'));         }
else { $UPD_INTERVAL = $UPD_INTERVAL;  } } else { $UPD_INTERVAL = quotemeta($query{'UPD_INTERVAL'});   }

if ( !$query{'logfile'} )   { if ( param('logfile')  ) { $logfile = quotemeta(param('logfile'));         }
else { $logfile = $logfile;  } } else { $logfile = quotemeta($query{'logfile'});   }

if ( !$query{'rawfile'} )   { if ( param('rawfile')  ) { $rawfile = quotemeta(param('rawfile'));         }
else { $rawfile = $rawfile;  } } else { $rawfile = quotemeta($query{'rawfile'});   }

if ( !$query{'sndfile'} )   { if ( param('sndfile')  ) { $sndfile = quotemeta(param('sndfile'));         }
else { $sndfile = $sndfile;  } } else { $sndfile = quotemeta($query{'sndfile'});   }

if ( !$query{'enabled'} )   { if ( param('enabled')  ) { $enabled = quotemeta(param('enabled'));         }
else { $enabled = $enabled;  } } else { $enabled = quotemeta($query{'enabled'});   }

if ( !$query{'EXT_REM'} )   { if ( param('EXT_REM')  ) { $EXT_REM = quotemeta(param('EXT_REM'));         }
	else { $EXT_REM = $EXT_REM;  } } else { $EXT_REM = quotemeta($query{'EXT_REM'});   }

if ( param('getAPIKey') )   {
  #print STDERR "getting API-Key!\n";
  $API_KEY = trim(qx(python3 $LBPDATA/$lbpplugindir/miyoplugin.py -getAPIKey));
}

if ( param('getCubeIP') )   {
  #print STDERR "getting getCubeIP!\n";
  #$MIYO_IP = quotemeta(qx(python3 $LBPDATA/$lbpplugindir/miyoplugin.py -getCubeIP));
  $MIYO_IP = trim(qx(python3 $LBPDATA/$lbpplugindir/miyoplugin.py -getCubeIP));
}

if ( param('getLBIP') )   {
  #print STDERR "getting getLBIP!\n";
  $LB_IP = $lbip;
}

if ( param('restartPlugin') )   {
  print STDERR "restarting MIYOplugin-service!\n";
  qx($LBPBIN/$lbpplugindir/service.sh restart);
}

if ( param('doVorlage') )   {
  $LOX_IP = unquotemeta($LOX_IP);
  # Speicherort fuer die Datei muss downloadbar sein
  my $vdatei = "MIYOplugin-in-out.xml";
  open(my $fh, ">", "$vdatei") || die "can not write to $vdatei";
  #print $fh "$lbip $LB_PORT ".unquotemeta($LOX_IP)." $LOX_PORT\n";
  my $cname = "";
  my $line = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
  $line .= "<VirtualInUdp Title=\"MIYOplugin\" Comment=\"\" Address=\"\" Port=\"$LOX_PORT\">\n";
  print $fh $line;
  # hier die Input-Ports
  my $mSID = "MIYOcentral";
  my @sys_cmd = ("running","updated","forecast","temperature");
  foreach my $cmd ( @sys_cmd ) {
    print $fh "\t<VirtualInUdpCmd Title=\"MIYO-$cmd\" Comment=\"MIYO-$cmd\" Address=\"\" Check=\"SID=$mSID\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
  }
  my @k_cmd = ("irrigationNextStart","irrigationNextEnd","irrigation","externBlock","automaticMode");
  my @v_cmd = ("valveStatus","lastIrrigationStart","lastIrrigationEnd","lastIrrigationDuration","openValve");
  my @s_cmd = ("moisture","brightness","temperature","irrigationNecessary","irrigationPossible");
  my @b_cmd = ("lastUpdate","rssi","reachable","charging","chargingDurationDay","solarVoltage");
  for my $i (1..$maxdev) {
    if ($i < 10) {$marker = "c_0${i}_0${i}"} else {$marker = "c_${i}_${i}"};
    if (defined ${$marker}) {
      $cname = encode("UTF-8", ${$marker});
      #print STDERR "$marker existiert und hat den Wert ${$marker} - $cname\n";
      foreach my $cmd ( @k_cmd ) {
        print $fh "\t<VirtualInUdpCmd Title=\"$cname-$cmd\" Comment=\"MIYO-$cmd $cname\" Address=\"\" Check=\"SID=$cname\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
      }
    }
    for my $j (1..$maxdev) {
      if ($j < 10) {$marker = "v_0${i}_0${j}"} else {$marker = "v_${i}_${j}"};
      if (defined ${$marker}) {
        $cname = encode("UTF-8", ${$marker});
        #print STDERR "j: $j $marker ${$marker} - $cname\n";
        foreach my $cmd ( @v_cmd ) {
          #print STDERR "\"SID=$cname\\i$cmd=\\i\\v\"\n";
          print $fh "\t<VirtualInUdpCmd Title=\"$cname-$cmd\" Comment=\"MIYO-$cmd $cname\" Address=\"\" Check=\"SID=$cname\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
        }
        foreach my $cmd ( @b_cmd ) {
          #print STDERR "\"SID=${$marker}\\i$cmd=\\i\\v\"\n";
          print $fh "\t<VirtualInUdpCmd Title=\"$cname-$cmd\" Comment=\"MIYO-$cmd $cname\" Address=\"\" Check=\"SID=$cname\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
        }
      }
    }
    for my $j (1..$maxdev) {
      if ($j < 10) {$marker = "s_0${i}_0${j}"} else {$marker = "s_${i}_${j}"};
      if (defined ${$marker}) {
        $cname = encode("UTF-8", ${$marker});
        #print STDERR "j: $j $marker ${$marker} - $cname\n";
        foreach my $cmd ( @s_cmd ) {
          #print STDERR "\"SID=${$marker}\\i$cmd=\\i\\v\"\n";
          print $fh "\t<VirtualInUdpCmd Title=\"$cname-$cmd\" Comment=\"MIYO-$cmd $cname\" Address=\"\" Check=\"SID=$cname\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
        }
        foreach my $cmd ( @b_cmd ) {
          #print STDERR "\"SID=${$marker}\\i$cmd=\\i\\v\"\n";
          print $fh "\t<VirtualInUdpCmd Title=\"$cname-$cmd\" Comment=\"MIYO-$cmd $cname\" Address=\"\" Check=\"SID=$cname\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
        }
      }
    }
  }
  $line = "</VirtualInUdp>\n\n";
  # hier der Block fuer die virtuellen Outs
  $line .= "<VirtualOut Title=\"MIYOplugin\" Comment=\"\" Address=\"/dev/udp/$lbip/$LB_PORT\" CmdInit=\"\" CloseAfterSend=\"true\" CmdSep=\" \">\n";
  print $fh $line;
  # hier die Output-Ports
  @k_cmd = (["circuit","Bewässerung","Circuit.irrigation,mode=start;duration=90","Circuit.irrigation,mode=stop","false"],
        ["irrigationStart","Start Bewässerung","Circuit.irrigation,mode=start;duration=90","","false"],
        ["irrigationStop","Stop Bewässerung","Circuit.irrigation,mode=stop","","false"],
        ["automaticMode","AutomaticMode","Circuit.edit,automaticMode=true","Circuit.edit,automaticMode=false","false"],
        ["block","ExternBlock","Extern.block,block=true","Extern.block,block=false","false"],
        ["considerMower","considerMower","Circuit.edit,considerMower=true","Circuit.edit,considerMower=false","false"]
                );
                 #    Titel,comment,start,stop,Analog
  @s_cmd = (["MIYO-reboot","MIYO-Reboot","SID=MIYOplugin,System.reboot","","false"],
            ["MIYO-externMower","MIYO-externMower","SID=MIYOplugin,Extern.mower,mower=<v>","SID=MIYOplugin,Extern.mower,mower=<v>","false"],
            ["MIYO-externRain","MIYO-externRain","SID=MIYOplugin,Extern.rain,rain=<v>","SID=MIYOplugin,Extern.rain,rain=<v>","false"],
            ["MIYO-externTemperature","MIYO-externTemperature","SID=MIYOplugin,Extern.temperature,temperature=<v>","SID=MIYOplugin,Extern.temperature,temperature=<v>","true"],
            ["MIYO-externWind","MIYO-externWind","SID=MIYOplugin,Extern.wind,wind=<v>","SID=MIYOplugin,Extern.wind,wind=<v>","true"],
           );
    foreach my $ocmd ( @s_cmd ) {
    print $fh "\t<VirtualOutCmd Title=\"${$ocmd}[0]\" Comment=\"${$ocmd}[1]\" CmdOnMethod=\"GET\" CmdOffMethod=\"GET\" CmdOn=\"${$ocmd}[2]\" CmdOnHTTP=\"\" CmdOnPost=\"\" CmdOff=\"${$ocmd}[3]\" CmdOffHTTP=\"\" CmdOffPost=\"\" Analog=\"${$ocmd}[4]\" Repeat=\"0\" RepeatRate=\"0\"/>\n";
  }
  for my $i (1..$maxdev) {
    if ($i < 10) {$marker = "c_0${i}_0${i}"} else {$marker = "c_${i}_${i}"};
    if (defined ${$marker}) {
      $cname = encode("UTF-8", ${$marker});
      foreach my $ocmd ( @k_cmd ) {
        print $fh "\t<VirtualOutCmd Title=\"$cname-${$ocmd}[0]\" Comment=\"${$ocmd}[1]\" CmdOnMethod=\"GET\" CmdOffMethod=\"GET\" CmdOn=\"SID=MIYOplugin,${$ocmd}[2],$cname\" CmdOnHTTP=\"\" CmdOnPost=\"\" CmdOff=\"SID=MIYOplugin,${$ocmd}[3],$cname\" CmdOffHTTP=\"\" CmdOffPost=\"\" Analog=\"false\" Repeat=\"0\" RepeatRate=\"0\"/>\n";
      }
    }
  }

  $line = "</VirtualOut>\n";
  print $fh $line;
  close($fh);
  my $filesize = -s "$vdatei";
  # jetzt irgendwie einen Download initiieren
  $linkvorlage ="<a id=\"autoload\" href=\"$vdatei\" download=\"$vdatei\" type=\"application/octet-stream\"></a>";
}

if ( param('getCircuit') )   {
  #print STDERR "get Circuits!\n";
  # so funktioniert es auch - ueber ENV erscheint es mir aber "sauberer"
  #my $cir_str = qx(PYTHONIOENCODING=UTF-8 python3 $LBPDATA/$lbpplugindir/miyoplugin.py -getCircuit);
  $ENV{'PYTHONIOENCODING'} = 'UTF-8';
  my $cir_str = qx(python3 $LBPDATA/$lbpplugindir/miyoplugin.py -getCircuit);

  #my $cir_str = "";
  #open (DATEI, "/tmp/problem-umlaute.json") or die $!;
  #  while(<DATEI>){
  #    $cir_str = $cir_str.$_;
  #  }
  #close (DATEI);

  #print STDERR "*$cir_str*\n";

  my $i = 0;
  my $hashref = eval { decode_json($cir_str)->{params}->{circuits} };
  #print STDERR "$@\n";
  if ($@ eq "") {
    $conf = new Config::Simple("$home/config/plugins/$lbpplugindir/miyoplugin.conf");
    #print STDERR "$home/config/plugins/$lbpplugindir/miyoplugin.conf\n";
    for my $circuit (sort keys %$hashref){
      $i++;
      my $j = 0;
      my $cname = quotemeta($hashref->{$circuit}{name}); # Hecke
      # replace all spaces with "_"
      #my $cmiyo = $cname;
      #$cname =~ tr/ /_/;
      my $sid = quotemeta($hashref->{$circuit}{sensor}); # ID des Sensors
      my $sipv6 = quotemeta($hashref->{$circuit}->{sensorData}->{ipv6});
      #print STDERR "$i: cid: $circuit name: $cname sensor: $sid\n";
      $conf->param("Circuit-".$i."\.circuitid", unquotemeta("${circuit}"));
      $conf->param("Circuit-".$i."\.circuitname", unquotemeta("${cname}"));
      $conf->param("Circuit-".$i."\.circuitmiyo", unquotemeta("${cname}"));
      #$conf->param("Circuit-".$i."\.circuitmiyo", unquotemeta("${cmiyo}"));
      $conf->param("Circuit-".$i."\.sensorid-1", unquotemeta("${sid}"));
      $conf->param("Circuit-".$i."\.sensorname-1", unquotemeta("${cname}-S1"));
      $conf->param("Circuit-".$i."\.sensoripv6-1", unquotemeta("${sipv6}"));
      if ($i < 10) {$marker = "c_0${i}_0${i}"} else {$marker = "c_${i}_${i}"};
      undef ${$marker};
      my $valves = $hashref->{$circuit}->{valves};
      for my $valve (sort keys %$valves){
        $j++;
        my $vid = quotemeta($valves->{$valve}->{valve});
        my $vipv6 = quotemeta($valves->{$valve}->{valveData}{ipv6});
        #print STDERR "$j: valveid: $vid\n";
        $conf->param("Circuit-".$i."\.valveid-$j", unquotemeta("${vid}"));
        $conf->param("Circuit-".$i."\.valvename-$j", unquotemeta("${cname}-V$j"));
        $conf->param("Circuit-".$i."\.valveipv6-$j", unquotemeta("${vipv6}"));
        if ($i < 10) {$marker = "s_0${i}_"} else {$marker = "s_${i}_"};
        if ($j < 10) {$marker .= "0${j}"} else {$marker .= "${j}"};
        undef ${$marker};
      }
    }
    $conf->save();
  }
}

if ( param('CheckLBPort') )   {
  our $udpok = "";
  #print STDERR "*OLI*: ".$LBPDATA."/".$lbpplugindir."/miyoplugin.py -checkLBPort ".$LB_PORT."\n";
  #print STDERR "*OLI*: $LB_PORT\n";
  $udpok = trim(qx(python3 $LBPDATA/$lbpplugindir/miyoplugin.py -checkLBPort $LB_PORT));
  #print STDERR "*OLI*: *".$udpok."*\n";
  if ($udpok eq "ok") {
    $LB_PORT = $LB_PORT;
  } else {
    $LB_PORT = 0;
  }
  print STDERR "check UDP Port ".$LB_PORT." if usable: ".$udpok."\n";
}

if (defined $MIYO_IP) { debugPrint("  MIYO_IP: ".$MIYO_IP); }
if (defined $LOX_IP) { debugPrint("  LOX_IP: ".$LOX_IP); }
if (defined $LOX_PORT) { debugPrint("  LOX_PORT: ".$LOX_PORT); }
if (defined $LB_IP) { debugPrint("  LB_IP: ".$LB_IP); }
if (defined $LB_PORT) { debugPrint("  LB_PORT: ".$LB_PORT); }
if (defined $API_KEY) { debugPrint("  API_KEY: ".$API_KEY); }
if (defined $LOX_TIME) { debugPrint("  LOX_TIME: ".$LOX_TIME); }
if (defined $UPD_INTERVAL) { debugPrint("  UPD_INTERVAL: ".$UPD_INTERVAL); }
if (defined $logfile) { debugPrint("  logfile: ".$logfile); }
if (defined $rawfile) { debugPrint("  rawfile: ".$rawfile); }
if (defined $sndfile) { debugPrint("  sndfile: ".$sndfile); }
if (defined $enabled) { debugPrint("  enabled: ".$enabled); }
if (defined $EXT_REM) { debugPrint("  EXT_REM: ".$EXT_REM); }
debugPrint("parse parameters coming in done");

$miniserver = "";
if ($lang eq "de") {
  $helptext = "<b>grundsätzlicher Ablauf:</b><br /><br />1. hier die erforderlichen Daten eingeben<br /><br />2. Settings abspeichern mit <strong>Speichern</strong><br /><br />3. ggf. über <strong>CubeConf</strong> die Bewässerungskreise einlesen<br /><br />4. mit <strong>Restart</strong> das Plugin mit den neuen Settings neustarten<br /><br />5. optional per <strong>Vorlage</strong> die Vorlagen-Datei für Loxone downloaden<br /><br /><br /><b>Speichern</b><br />speichert die Config-Datei<br /><br /><b>CubeConf</b><br />Einlesen der Cube-Konfiguration<br /><br /><b>Restart</b><br />Neustart des MIYOplugin mit den aktualisierten Settings<br /><br /><b>Vorlage</b><br />Download der Loxone-Vorlagen-Datei<br /><br />";
} else {
  $helptext = "<b>Basic procedure:</b><br /><br />1. enter the required data here<br /><br />2. Save settings with <strong>Save</strong><br /><br />3. if necessary use <strong>CubeConf</strong> to read in the irrigation circuits.<br /><br />4. with <strong>Restart</strong> restart the plugin with the new settings<br /><br />5. optional<strong>template</strong> download the template file for Loxone<br /><br /><b>Save</b><br />saves the config file<br /><br /><b>CubeConf</b><br />Read the cube configuration<br /><br /><b>Restart</b><br />Restart the MIYOplugin with the updated settings<br /><br /><b>Template</b><br />Download the Loxone template file<br /><br />";
}
$helplink = "http://miyoplugin.phantasoft.de";
$restartMsg = "";


# ---------------------------------------
# Save settings to config file
# ---------------------------------------
debugPrint("save settings start");
if (param('savedata')) {
  #print STDERR "n. savedata in index\n";
  my $cfgfile = "$home/config/plugins/$lbpplugindir/miyoplugin.conf";
  $conf = new Config::Simple($cfgfile);

  $MIYO_IP = encode_entities($MIYO_IP);

  $conf->param('Config.MIYO_IP', unquotemeta($MIYO_IP));
  $conf->param('Config.LOX_IP', unquotemeta($LOX_IP));
  $conf->param('Config.LOX_PORT', unquotemeta($LOX_PORT));
  $conf->param('Config.LB_IP', unquotemeta($LB_IP));
  $conf->param('Config.LB_PORT', unquotemeta($LB_PORT));
  $conf->param('Config.API_KEY', unquotemeta($API_KEY));
  $conf->param('Config.LOX_TIME', unquotemeta($LOX_TIME));
  $conf->param('Config.UPD_INTERVAL', unquotemeta($UPD_INTERVAL));
  $conf->param('Config.ENABLED', unquotemeta($enabled));
  $conf->param('Config.EXT_REM', unquotemeta($EXT_REM));
  $conf->param('Logging.logfile', unquotemeta($logfile));
  $conf->param('Logging.rawfile', unquotemeta($rawfile));
  $conf->param('Logging.sndfile', unquotemeta($sndfile));

  # Oliver - hier die Bewaesserungskreise in die Config schreiben
  # die Arrays enthalten aber zu diesem Zeitpunkt nur die Initialwerte!
  # erneut erzeugen und entsprechend Config schreiben
  for my $i (1..$maxdev) {
    my $circuit = "Circuit\-".$i;
    my $cid = "";
    if ($i < 10) {$marker = "c_0${i}_0${i}"} else {$marker = "c_${i}_${i}"};
    if (defined ${$marker}) {
      $cid = clearString(encode_entities($conf->param($circuit."\.circuitid")));
      #print STDERR "$i: $cid\n";
    }
    if ($cid ne "") {
      #my $cname = clearString(encode_entities($conf->param($circuit."\.circuitname")));
      my $cname = clearString(decode_entities($conf->param($circuit."\.circuitname")));

      if ($i < 10) {$marker = "c_0${i}_0${i}"} else {$marker = "c_${i}_${i}"};
      if (defined ${$marker}) {
        #print STDERR "$i: $circuit encode_entities($cname) $cid ${$marker}\n";
	#print STDERR "C: $cname"," U: ",unquotemeta($cname)," D: ",decode_entities($cname),"\n";
        $conf->param($circuit."\.circuitname", unquotemeta(${$marker}));
      }
      for my $j (1..$maxdev) {
        my $valve = "valveid\-".$j;
        my $vid = clearString(encode_entities($conf->param($circuit."\.$valve")));
        $valve = "valvename\-".$j;
        my $vname = clearString(encode_entities($conf->param($circuit."\.$valve")));
        if ($vid ne "") {
          if ($i < 10) {$marker = "v_0${i}_"} else {$marker = "v_${i}_"};
          if ($j < 10) {$marker .= "0${j}"} else {$marker .= "${j}"};
          if (defined ${$marker}) {
            #print STDERR "$i: $circuit $valve $vid ${$marker}\n";
            $conf->param($circuit."\.$valve", unquotemeta(${$marker}));
          }
        }
        my $sensor = "sensorid\-".$j;
        my $sid = clearString(encode_entities($conf->param($circuit."\.$sensor")));
        $sensor = "sensorname\-".$j;
        my $sname = clearString(encode_entities($conf->param($circuit."\.$sensor")));
        if ($sid ne "") {
          if ($i < 10) {$marker = "s_0${i}_"} else {$marker = "s_${i}_"};
          if ($j < 10) {$marker .= "0${j}"} else {$marker .= "${j}"};
          if (defined ${$marker}) {
            #print STDERR "$i: $circuit $sensor $sid ${$marker}\n";
            $conf->param($circuit."\.$sensor", unquotemeta(${$marker}));
          }
        }
      }
    }
  }
  $conf->save();
  debugPrint("  settings saved to $cfgfile");
}
debugPrint("save settings done");

# ---------------------------------------
# Parse config file
# ---------------------------------------
debugPrint("parse config file start");
my $cfgfile = "$home/config/plugins/$lbpplugindir/miyoplugin.conf";
if (-e $cfgfile) {
  #print STDERR "$cfgfile found!\n";
  $conf = new Config::Simple("$cfgfile");
} else {
  #print STDERR "$cfgfile not found!\n";
  # hier den default-dummy kreieren
  $conf = new Config::Simple(syntax=>'ini');
  $conf->param('Config.MIYO_IP',$MIYO_IP);
  $conf->param('Config.LB_PORT',$LB_PORT);
  $conf->param('Config.UPD_INTERVAL','10');
  $conf->write($cfgfile);
}

#$conf = new Config::Simple("$home/config/plugins/$lbpplugindir/miyoplugin.conf");
#print STDERR "OLI: *$conf*\n";
$MIYO_IP = clearString(encode_entities($conf->param('Config.MIYO_IP')));
$LOX_IP = clearString(encode_entities($conf->param('Config.LOX_IP')));
$LOX_PORT = clearString(encode_entities($conf->param('Config.LOX_PORT')));
$LB_IP = clearString(encode_entities($conf->param('Config.LB_IP')));
$LB_PORT = clearString(encode_entities($conf->param('Config.LB_PORT')));
$API_KEY = clearString(encode_entities($conf->param('Config.API_KEY')));
$LOX_TIME = clearString(encode_entities($conf->param('Config.LOX_TIME')));
$UPD_INTERVAL = clearString(encode_entities($conf->param('Config.UPD_INTERVAL')));
$logfile = clearString(encode_entities($conf->param('Logging.logfile')));
$rawfile = clearString(encode_entities($conf->param('Logging.rawfile')));
$sndfile = clearString(encode_entities($conf->param('Logging.sndfile')));
$enabled = clearString(encode_entities($conf->param('Config.ENABLED')));
$EXT_REM = clearString(encode_entities($conf->param('Config.EXT_REM')));

# Einlesen der Cube-Config
$CubeConfig = "<hr><table class=\"formtable\" border=\"0\" width=\"100%\">\n";
for my $i (1..$maxdev) {
  my $circuit =  "Circuit\-".$i;
  my $cid = clearString(encode_entities($conf->param($circuit."\.circuitid")));
  if ($cid ne "") {
    my $cname = clearString(encode_entities($conf->param($circuit."\.circuitname")));
    my $cmiyo = clearString(encode_entities($conf->param($circuit."\.circuitmiyo")));
    if ($i < 10) {$marker = "c_0${i}_0${i}"} else {$marker = "c_${i}_${i}"};
    #print STDERR "marker: $marker\n";
    ${$marker} = $cname;
    @kreis = [$i,$circuit,$cname,$cid,$marker];
    push (@kreise,@kreis);
    #print STDERR "Kreis $i circuit: $circuit\ncname: $cname\ncid:  $cid\nmarker: $marker\n ddmarker: ${$marker}\n";
    my $sp = "        ";
    $CubeConfig .= "<tr><td>&nbsp;</td></tr>";
    if ($lang eq "de") {
      $CubeConfig .= "<tr><td><b>Kreis $i ($cmiyo)</b></td></tr>";
      $CubeConfig .= "<tr><td width=\"25%\">Loxone-Name für $circuit:</td><td width=\"50%\"><input id=\"$marker\" name=\"$marker\" type=\"text\" class=\"textfield\" value=\"$cname\" data-validation-error-msg=\"* Please enter a name for this circuit\"></td><td style=\"width: 10px;\"><div class=\"tooltip\">?<span class=\"tooltiptext\">&Uuml;ber diesen Namen wird der Bew&auml;sserungskreis mit der ID $cid von Loxone identifiziert.<br /><strong>Es sollten keine Leer- oder Sonderzeichen enthalten sein!</strong></span></div></td><td width=\"25%\">&nbsp;</td></tr>";
    } else {
      $CubeConfig .= "<tr><td><b>Circuit $i ($cmiyo)</b></td></tr>";
      $CubeConfig .= "<tr><td width=\"25%\">Loxone-Name for $circuit:</td><td width=\"50%\"><input id=\"$marker\" name=\"$marker\" type=\"text\" class=\"textfield\" value=\"$cname\" data-validation-error-msg=\"* Please enter a name for this circuit\"></td><td style=\"width: 10px;\"><div class=\"tooltip\">?<span class=\"tooltiptext\">With this name Loxone identifies the irrigation circuit with ID $cid.<br /><strong>There should be no spaces or special characters!</strong></span></div></td><td width=\"25%\">&nbsp;</td></tr>";
    }
    for my $j (1..$maxdev) {
      my $valve = "valveid\-".$j;
      my $vid = clearString(encode_entities($conf->param($circuit."\.$valve")));
      $valve = "valvename\-".$j;
      my $vname = clearString(encode_entities($conf->param($circuit."\.$valve")));
      my $vipid = "valveipv6\-".$j;
      my $vipv6 = clearString(encode_entities($conf->param($circuit."\.$vipid")));
      $vipid = substr($vipv6,15,7) if $vipv6 ne "";
      #print STDERR "--> $valve $vid *$vipv6* *$vipid* $vname\n" if $vid ne "";
      if ($vid ne "") {
        if ($i < 10) {$marker = "v_0${i}_"} else {$marker = "v_${i}_"};
        if ($j < 10) {$marker .= "0${j}"} else {$marker .= "${j}"};
        ${$marker} = $vname;
        @ventil = [$i,$valve,$vname,$vid,$marker];
        push (@ventile,@ventil);
        if ($lang eq "de") {
          $CubeConfig .= "<tr><td width=\"25%\">Loxone-Name für $valve:</td><td width=\"50%\"><input id=\"$marker\" name=\"$marker\" type=\"text\" class=\"textfield\" value=\"$vname\" data-validation-error-msg=\"* Please enter a name for this valve\"></td><td style=\"width: 10px;\"><div class=\"tooltip\">?<span class=\"tooltiptext\">&Uuml;ber diesen Namen wird das Ventil mit der ID $vipid von Loxone identifiziert.<br /><strong>Es sollten keine Leer- oder Sonderzeichen enthalten sein!</strong></span></div></td><td width=\"25%\">&nbsp;</td></tr>";
        } else {
          $CubeConfig .= "<tr><td width=\"25%\">Loxone-Name for $valve:</td><td width=\"50%\"><input id=\"$marker\" name=\"$marker\" type=\"text\" class=\"textfield\" value=\"$vname\" data-validation-error-msg=\"* Please enter a name for this valve\"></td><td style=\"width: 10px;\"><div class=\"tooltip\">?<span class=\"tooltiptext\">With this name Loxone identifies the valve with ID $vipid.<br /><strong>There should be no spaces or special characters!</strong></span></div></td><td width=\"25%\">&nbsp;</td></tr>";
        }
      }
# Oliver: zur huebscheren Sortierung ggf. zweimal bis 10 zaehlen - marker pruefen!
    }
    for my $j (1..$maxdev) {
      my $sensor = "sensorid\-".$j;
      my $sid = clearString(encode_entities($conf->param($circuit."\.$sensor")));
      $sensor = "sensorname\-".$j;
      my $sname = clearString(encode_entities($conf->param($circuit."\.$sensor")));
      my $sipid = "sensoripv6\-".$j;
      my $sipv6 = clearString(encode_entities($conf->param($circuit."\.$sipid")));
      $sipid = substr($sipv6,15,7) if $sipv6 ne "";
      #print STDERR "--> $sensor $sid *$sipv6* *$sipid* $sname\n" if $sid ne "";
      if ($sid ne "") {
        if ($i < 10) {$marker = "s_0${i}_"} else {$marker = "s_${i}_"};
        if ($j < 10) {$marker .= "0${j}"} else {$marker .= "${j}"};
        ${$marker} = $sname;
        @sensor = [$i,$sensor,$sname,$sid,$marker];
        push (@sensoren,@sensor);
        if ($lang eq "de") {
          $CubeConfig .= "<tr><td width=\"25%\">Loxone-Name für $sensor:</td><td width=\"50%\"><input id=\"$marker\" name=\"$marker\" type=\"text\" class=\"textfield\" value=\"$sname\" data-validation-error-msg=\"* Please enter a name for this sensor\"></td><td style=\"width: 10px;\"><div class=\"tooltip\">?<span class=\"tooltiptext\">&Uuml;ber diesen Namen wird der Sensor mit der ID $sipid von Loxone identifiziert.<br /><strong>Es sollten keine Leer- oder Sonderzeichen enthalten sein!</strong></span></div></td><td width=\"25%\">&nbsp;</td></tr>";
        } else {
          $CubeConfig .= "<tr><td width=\"25%\">Loxone-Name for $sensor:</td><td width=\"50%\"><input id=\"$marker\" name=\"$marker\" type=\"text\" class=\"textfield\" value=\"$sname\" data-validation-error-msg=\"* Please enter a name for this sensor\"></td><td style=\"width: 10px;\"><div class=\"tooltip\">?<span class=\"tooltiptext\">With this name Loxone identifies the sensor with ID $sipid.<br /><strong>There should be no spaces or special characters!</strong></span></div></td><td width=\"25%\">&nbsp;</td></tr>";
        }
      }
    }
  }
}
$CubeConfig .= "</table>";


# ---------------------------------------
# Set Enabled / Disabled switch
# ---------------------------------------
if ($enabled eq "True") {
  $enabled = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>\n";
  qx($LBPBIN/$lbpplugindir/service.sh start);
} else {
  $enabled = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>\n";
  qx($LBPBIN/$lbpplugindir/service.sh stop);
}

# ---------------------------------------
# Set LOX_TIME Enabled / Disabled switch
# ---------------------------------------
if ($LOX_TIME eq "True") {
  $LOX_TIME = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>\n";
} else {
  $LOX_TIME = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>\n";
}

# ---------------------------------------
# Set EXT_REM Enabled / Disabled switch
# ---------------------------------------
if ($EXT_REM eq "True") {
  $EXT_REM = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>\n";
  } else {
    $EXT_REM = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>\n";
}
     
# ---------------------------------------
# Fill Miniserver selection dropdown
# ---------------------------------------
if ($lang eq "de") {
  $MSselectlist .= '<option value="">Loxone-Wahl</option>'."\n        ";
} else {
  $MSselectlist .= '<option value="">select Loxone</option>'."\n        ";
}
for (my $i = 1; $i <= $cfg->param('BASE.MINISERVERS');$i++) {
  $MSselectlist .= '<option value="'.$cfg->param("MINISERVER$i.IPADDRESS").'">'.$cfg->param("MINISERVER$i.IPADDRESS")."</option>";
}
if (defined $MIYO_IP) { debugPrint("  MIYO_IP: ".$MIYO_IP); }
if (defined $LOX_IP) { debugPrint("  LOX_IP: ".$LOX_IP); }
if (defined $LOX_PORT) { debugPrint("  LOX_PORT: ".$LOX_PORT); }
if (defined $LB_IP) { debugPrint("  LB_IP: ".$LB_IP); }
if (defined $LB_PORT) { debugPrint("  LB_PORT: ".$LB_PORT); }
if (defined $API_KEY) { debugPrint("  API_KEY: ".$API_KEY); }
if (defined $LOX_TIME) { debugPrint("  LOX_TIME: ".$LOX_TIME); }
if (defined $UPD_INTERVAL) { debugPrint("  UPD_INTERVAL: ".$UPD_INTERVAL); }
if (defined $logfile) { debugPrint("  logfile: ".$logfile); }
if (defined $rawfile) { debugPrint("  rawfile: ".$rawfile); }
if (defined $sndfile) { debugPrint("  sndfile: ".$sndfile); }
if (defined $enabled) { debugPrint("  enabled: ".$enabled); }
if (defined $EXT_REM) { debugPrint("  EXT_REM: ".$EXT_REM); }
debugPrint("parse config file done");

# Init Language
# Clean up lang variable
debugPrint("init language start");
$lang         =~ tr/a-z//cd;
$lang         = substr($lang,0,2);
# If there's no language phrases file for choosed language, use "en" as default
if (!-e "$installfolder/templates/system/$lang/language.dat") {
  $lang = "en";
}

# Read translations / phrases
$languagefile           = "$installfolder/templates/system/$lang/language.dat";
$phrase                 = new Config::Simple($languagefile);
$languagefileplugin     = "$installfolder/templates/plugins/$lbpplugindir/$lang/language.dat";
$phraseplugin           = new Config::Simple($languagefileplugin);
debugPrint("  lang: ".$lang);
debugPrint("init language done");

# Title
$template_title = "$prgname v$prgver";

# ---------------------------------------
# Load header and replace HTML Markup <!--$VARNAME--> with perl variable $VARNAME
# ---------------------------------------
debugPrint("load header file start");
open(F,"$installfolder/templates/system/$lang/header.html") || die "Missing template system/$lang/header.html";
while (<F>) {
  $_ =~ s/<!--\$(.*?)-->/${$1}/g;
  print $_;
}
close(F);
debugPrint("load header file done");

# ---------------------------------------
# Load content from template
# ---------------------------------------
debugPrint("load content file start");
#open(F,"$installfolder/templates/plugins/$lbpplugindir/$lang/content.html") || die "Missing template $lang/content.html";
open(F,"$installfolder/templates/plugins/$lbpplugindir/$lang/content.html") || open(F,"$installfolder/templates/plugins/$lbpplugindir/en/content.html") || die "Missing template $lang/content.html";
while (<F>) {
  $_ =~ s/<!--\$(.*?)-->/${$1}/g;
  print $_;
}
close(F);
debugPrint("load content file done");

# ---------------------------------------
# Load footer and replace HTML Markup <!--$VARNAME--> with perl variable $VARNAME
# ---------------------------------------
debugPrint("load footer file start");
open(F,"$installfolder/templates/system/$lang/footer.html") || die "Missing template system/$lang/header.html";
while (<F>) {
  $_ =~ s/<!--\$(.*?)-->/${$1}/g;
  print $_;
}
close(F);
debugPrint("load footer file done");

debugPrint("cgi exit");
exit;

